import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_media/model/nui_media_file.dart';
import 'package:nui_media/nui_media.dart';

NUIDialog showMediaPagerPreview(BuildContext context, {@required List<NUIMediaFile> files, int initialPage: 0}){
  final dialog = showNUIDialog(context: context, content: (context){
    return NUIMediaPagerPreviewUI(files: files, initialPage: initialPage);
  }, transition: PageTransition.BLUR_BACKGROUND);
  return dialog;
}

class NUIMediaPagerPreviewUI extends NUIScreen {
  final List<NUIMediaFile> files;
  final int initialPage;

  NUIMediaPagerPreviewUI({Key key, @required this.files, this.initialPage}) : super(
      content : (context) => NUIMediaPagerView(files: files, initialPage: initialPage), screenCode: "NUIMediaPagerPreviewUI", key: key, overlayStyle: NUIOverlayStyle(darkStatusBarIcon: false));
}

class NUIMediaPagerView extends StatefulWidget {
  NUIMediaPagerView({@required this.files, this.initialPage : 0});
  final List<NUIMediaFile> files;
  final int initialPage;
  @override
  _NUIMediaPagerViewState createState() => _NUIMediaPagerViewState();
}

class _NUIMediaPagerViewState extends State<NUIMediaPagerView> {
  NUIListViewBloc<NUIMediaPagerItem, NUIMediaPagerItem> bloc;
  int _currentPage;
  @override
  void initState(){
    super.initState();
    _currentPage = widget.initialPage ?? 0;
    bloc = NUIListViewBloc<NUIMediaPagerItem, NUIMediaPagerItem>(callbackMethod: (List<NUIMediaPagerItem> current, bool isFirst, bool append, NUIMediaPagerItem cursor) async{
      final images = widget.files.map((e) => e.thumbData == null ? NUIMediaPagerImage(e) : NUIMediaPagerVideo(e)).toList();
      logNUI("NUIMediaPreview", "Parse medias successful of size : ${images.length}");
      final result = ListResult<NUIMediaPagerItem, NUIMediaPagerItem>(data: images, success: true, message: "Parse medias successful of size : ${images.length}", cursor: null);
      return result;
    });
  }

  Widget _getPageIndicator(){
    return Visibility(
      visible: widget.files.length > 1,
      child: NUIShadowView(
        spreadRadius: 30,
        blurRadius: 3,
        opacity: 0.1,
        child: Text("${_currentPage + 1} / ${widget.files.length}", style: TextStyle(color: NUIColors.NUIWhite, fontSize: 12, fontWeight: FontWeight.bold))
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final statusBarHeight = NUIDisplayUtil.statusBarHeight(context);
    return Container(
        child: Stack(
          children: [
            NUIPageView(
                bloc: bloc,
                viewportFraction: 1.0,
                initialPosition: _currentPage,
                repo: NUIMediaListItemRepo(),
                initialList: List<NUIMediaPagerItem>(),
                clickListener: (data, position, action){

                },
                pageChangeListener: (page){
                  setState(() {
                    _currentPage = page;
                  });
                },
            ),
            Align(
              alignment: Alignment.topCenter,
              child: Container(
                height: NUIDisplayUtil.statusBarHeight(context) + 20,
                width: screenWidth(context),
                decoration: BoxDecoration(
                    gradient: NUILinearGradient(
                        colors: [
                          NUIColors.NUIShadowBlack.withOpacity(0.5),
                          NUIColors.NUIShadowBlack.withOpacity(0.15),
                          NUIColors.NUITransparent
                        ],
                        orientation: GradientOrientation.T_B
                    )
                ),
              ),
            ),
            Padding(
              padding: EdgeInsets.only(top: statusBarHeight),
              child: Align(
                alignment: Alignment.topLeft,
                child: NUIShadowView(
                  spreadRadius: 5,
                  blurRadius: 30,
                  opacity: 0.2,
                  child: Container(
                    margin: EdgeInsets.only(left: 5),
                    child: NUIRippleContainer(
                      normalColor: NUIColors.NUITransparent,
                      rippleColor: NUIColors.NUIBlack.withOpacity(0.2),
                      borderRadius: BorderRadius.circular(25),
                      child: Padding(
                        padding: EdgeInsets.all(10),
                        child: Icon(Icons.close, color: NUIColors.NUIWhite, size: 30)
                      ),
                      onTap: (){
                        NUINavigator.pop(context);
                      },
                    ),
                  ),
                ),
              ),
            ),
            Padding(
              padding: EdgeInsets.only(top: statusBarHeight),
              child: Align(
                alignment: Alignment.topRight,
                child: _getPageIndicator()
              ),
            )
          ],
        ),
    );
  }
}


class NUIImagePreviewContent extends NUIListStatefulWidget {
  NUIImagePreviewContent(this.file, {NUIListActionHandler handler}): super(handler: handler);
  final NUIMediaPagerImage file;

  @override
  _NUIImagePreviewContentState createState() => _NUIImagePreviewContentState();
}

class _NUIImagePreviewContentState extends State<NUIImagePreviewContent> {
  @override
  Widget build(BuildContext context) {
    final isLocalImage = widget.file.file.isLocal == true;
    logNUI("NUIMediaPreview", "Loading image from local : $isLocalImage");
    return Container(
      child: Center(
        child: InteractiveViewer(
          child: isLocalImage ? Image.file(widget.file.file.getFile()) : loadImage(widget.file.file.path),
          maxScale: 3.0,
        ),
      ),
    );
  }
}

class NUIVideoPreviewContent extends NUIListStatefulWidget {
  NUIVideoPreviewContent(this.file, {NUIListActionHandler handler}): super(handler: handler);
  final NUIMediaPagerVideo file;
  @override
  _NUIVideoPreviewContentState createState() => _NUIVideoPreviewContentState();
}

class _NUIVideoPreviewContentState extends State<NUIVideoPreviewContent> {
  @override
  Widget build(BuildContext context) {
    logNUI("NUIMediaPreview", "Loading video");
    return Container(
      child: Center(
        child: NUIVideoListViewPlayer(
            mediaFile: widget.file.file,
            autoPlay: true,
            playFraction: 0.8,
            aspectRatio: widget.file.file.widthToHeightRatio(),
            showButtons: true,
        ),
      ),
    );
  }
}
